/* eslint-env jest */
import '../../../../../.jest/__mocks__/matchMedia'; // Must be imported before the tested file
import { SldsInputRadioGroup } from '../input-radio-group';
import * as utils from '../../../../../.jest/utils';
// Register custom element for jest wrapper
customElements.define('slds-input-radio-group', SldsInputRadioGroup);

describe('slds-input-radio-group', () => {
  afterEach(() => {
    // The jsdom instance is shared across test cases in a single file so reset the DOM
    while (document.body.firstChild) {
      document.body.removeChild(document.body.firstChild);
    }
  });

  // Input Radio Group Tests
  it('contains 3 named parts with the values "input-radio-group", "legend", and "options"', async () => {
    const element = utils.renderIntoBody(`<slds-input-radio-group></slds-input-radio-group>`);
    const inputRadioGroup = element.shadowRoot.querySelector('[part~="input-radio-group"]');
    const legend = element.shadowRoot.querySelector('[part~="legend"]');
    const options = element.shadowRoot.querySelector('[part~="options"]');
    expect(inputRadioGroup).toBeInTheDocument();
    expect(legend).toBeInTheDocument();
    expect(options).toBeInTheDocument();
  });

  it('contains 1 named slots with the values "help-text"', async () => {
    const element = utils.renderIntoBody(`<slds-input-radio-group></slds-input-radio-group>`);
    const helptext = element.shadowRoot.querySelector('[name~="help-text"]');
    expect(helptext).toBeInTheDocument();
  });

  it('contains 1 part with the value "required", required indicator and attribute if required', async () => {
    const element = utils.renderIntoBody(`<slds-input-radio-group required></slds-input-radio-group>`);
    const elementControl = utils.renderIntoBody(`<slds-input-radio-group></slds-input-radio-group>`);
    const partRequired = element.shadowRoot.querySelector('[part~="required"]');
    const partRequiredControl = elementControl.shadowRoot.querySelector('[part~="required"]');
    const required = element.shadowRoot.querySelector('abbr[title~="required"]');
    const requiredControl = elementControl.shadowRoot.querySelector('abbr[title~="required"]');

    expect(partRequired).toBeInTheDocument();
    expect(partRequiredControl).toBeNull();
    expect(required).toHaveAttribute('title', 'required');
    expect(requiredControl).toBeNull();
  });

  it('sets the attribute "name" on host', async () => {
    const element = utils.renderIntoBody(`<slds-input-radio-group name="example"></slds-input-radio-group>`);

    expect(element).toHaveAttribute('name', 'example');
  });

  it('sets the attribute "orientation" on host and "aria-orientation" on fieldset', async () => {
    const element = utils.renderIntoBody(
      `<slds-input-radio-group orientation="vertical"></slds-input-radio-group>`,
    );
    const inputRadioGroup = element.shadowRoot.querySelector('[part~="input-radio-group"]');

    expect(element).toHaveAttribute('orientation', 'vertical');
    expect(inputRadioGroup).toHaveAttribute('aria-orientation', 'vertical');
  });

  it('sets the attribute "disabled" on host and "aria-orientation" on fieldset', async () => {
    const element = utils.renderIntoBody(
      `<slds-input-radio-group orientation="vertical"></slds-input-radio-group>`,
    );
    const inputRadioGroup = element.shadowRoot.querySelector('[part~="input-radio-group"]');

    expect(element).toHaveAttribute('orientation', 'vertical');
    expect(inputRadioGroup).toHaveAttribute('aria-orientation', 'vertical');
  });

  // Input Radio Group Variants Tests
  it('sets the slds class for "variant" "label-hidden" on legen', async () => {
    const element = utils.renderIntoBody(
      `<slds-input-radio-group variant="label-hidden"></slds-input-radio-group>`,
    );
    const legend = element.shadowRoot.querySelector('[part~="legend"]');

    expect(legend).toHaveClass('slds-assistive-text');
  });

  it('sets the slds class for "variant" "label-inline" on host', async () => {
    const element = utils.renderIntoBody(
      `<slds-input-radio-group variant="label-inline"></slds-input-radio-group>`,
    );

    expect(element).toHaveClass('slds-form-element_horizontal');
  });

  it('sets the slds class for "variant" "label-stacked" on host', async () => {
    const element = utils.renderIntoBody(
      `<slds-input-radio-group variant="label-stacked"></slds-input-radio-group>`,
    );

    expect(element).toHaveClass('slds-form-element_stacked');
  });

  // Input Radio Tests
  it('contains 4 named parts with the values "radio", "input-radio", "indicator" and "label"', async () => {
    const element = utils.renderIntoBody(`<slds-input-radio-group></slds-input-radio-group>`);
    element.options = [{ label: 'Example Label', value: 'Example Value' }];

    await utils.delay(2000); /* Delay because of race condition */

    const radio = element.shadowRoot.querySelector('[part~="radio"]');
    const inputRadio = element.shadowRoot.querySelector('[part~="input-radio"]');
    const indicator = element.shadowRoot.querySelector('[part~="indicator"]');
    const label = element.shadowRoot.querySelector('[part~="label"]');

    expect(radio).toBeInTheDocument();
    expect(inputRadio).toBeInTheDocument();
    expect(indicator).toBeInTheDocument();
    expect(label).toBeInTheDocument();
    expect(label.innerHTML).toEqual('Example Label');
  });

  it('sets the attribute "checked" on the radio specified with "selected-option"', async () => {
    const element = utils.renderIntoBody(
      `<slds-input-radio-group selected-option="day"></slds-input-radio-group>`,
    );
    element.options = [
      { label: 'Day', value: 'day' },
      { label: 'Night', value: 'night' },
    ];

    await utils.delay(2000); /* Delay because of race condition */

    const day = element.shadowRoot.querySelector('[part~="radio"][value="day"]');
    const night = element.shadowRoot.querySelector('[part~="radio"][value="night"]');

    expect(day).toHaveAttribute('checked', 'true');
    expect(night).not.toHaveAttribute('checked');
  });
});
